import tkinter as tk
from tkinter import messagebox
from tkinter.ttk import Combobox, Progressbar
from database import Database
from map import Map
from graph import Graph


class SpotifyCountryAnalyzerGUI:
    def __init__(self, data_file):
        self.root = tk.Tk()
        self.root.title("Spotify Country Analyzer")
        self.root.geometry("900x600")
        self.root.resizable(False, False)

        self.database = Database(data_file)
        self.country_selected = False

        self.map_frame = tk.Frame(self.root)
        self.map_frame.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)

        self.map = Map(self.map_frame, width=600, height=500)
        self.map.show()

        self.filter_frame = tk.Frame(self.root)
        self.filter_frame.pack(side=tk.LEFT, padx=10, pady=10)

        self.label_country = tk.Label(self.filter_frame, text="Country:")
        self.label_country.pack()

        self.combobox_country = Combobox(self.filter_frame, width=30)
        self.combobox_country.pack()

        self.button_filter_country = tk.Button(
            self.filter_frame,
            text="Filter Country",
            command=self.filter_country,
        )
        self.button_filter_country.pack(pady=10)

        self.button_plot_graph = tk.Button(
            self.filter_frame,
            text="Plot Graph (Locked)",
            state=tk.DISABLED,
            command=self.plot_graph
        )
        self.button_plot_graph.pack(pady=10)

        self.label_top_songs = tk.Label(
            self.filter_frame,
            text="Top Ten Songs:",
            font=("Helvetica", 12, "bold")
        )
        self.label_top_songs.pack(pady=10)

        self.text_top_songs = tk.Text(
            self.filter_frame,
            height=10,
            width=40,
            state=tk.DISABLED,
            font=("Helvetica", 10)
        )
        self.text_top_songs.pack()

        self.label_search_song = tk.Label(
            self.filter_frame,
            text="Search Song:",
            font=("Helvetica", 12, "bold")
        )
        self.label_search_song.pack(pady=10)

        self.entry_search_song = tk.Entry(
            self.filter_frame,
            width=30,
            font=("Helvetica", 10)
        )
        self.entry_search_song.pack()

        self.button_search = tk.Button(
            self.filter_frame,
            text="Search",
            command=self.search_songs,
            font=("Helvetica", 12)
        )
        self.button_search.pack(pady=10)

        self.progress_bar = Progressbar(
            self.filter_frame,
            orient='horizontal',
            mode='indeterminate',
            length=400,
            style='Custom.Horizontal.TProgressbar'
        )
        self.progress_bar.pack(pady=10)

        self.style = tk.ttk.Style()
        self.style.configure(
            'Custom.Horizontal.TProgressbar',
            thickness=15,
            troughcolor='white',
            background='green',
            troughrelief='flat',
            relief='flat'
        )

        self.combobox_country.bind('<<ComboboxSelected>>', self.unlock_plot_graph)

    def filter_country(self):
        country = self.combobox_country.get()
        self.progress_bar.start()

        # Simulating data loading process
        self.root.after(1000, self.load_data, country)

    def load_data(self, country):
        self.map.set_country(country)
        top_songs = self.database.top_ten_songs(country)
        self.text_top_songs.config(state=tk.NORMAL)
        self.text_top_songs.delete('1.0', tk.END)
        self.text_top_songs.insert(
            tk.END,
            f"Top Ten Songs in {country}:\n\n"
        )
        for i, (rank, song, artist, _) in enumerate(top_songs):
            self.text_top_songs.insert(
                tk.END,
                f"{rank}. {song} - {artist}\n"
            )
        self.text_top_songs.config(state=tk.DISABLED)
        self.progress_bar.stop()
        self.country_selected = True
        self.button_plot_graph.configure(text="Plot Graph", state=tk.NORMAL)

    def unlock_plot_graph(self, event):
        self.button_plot_graph.configure(text="Plot Graph (Locked)", state=tk.DISABLED)

    def plot_graph(self):
        if not self.country_selected:
            messagebox.showwarning("Warning", "Please select a country first.")
            return

        # Perform the plot graph operation
        country = self.combobox_country.get()
        graph = Graph(self.database.get_data())

        # Create a new window for displaying the graphs
        graph_window = tk.Toplevel(self.root)
        graph_window.title("Graphs")

        top_songs_by_rank_frame = tk.Frame(graph_window)
        top_songs_by_rank_frame.pack(side='top', fill='both', expand=True)
        graph.plot_top_songs_by_rank(country, top_songs_by_rank_frame)

        top_artists_by_streams_frame = tk.Frame(graph_window)
        top_artists_by_streams_frame.pack(side='top', fill='both', expand=True)
        graph.plot_top_artists_by_streams(country, top_artists_by_streams_frame)

        graph_window.mainloop()

    def search_songs(self):
        if not self.country_selected:
            messagebox.showwarning("Warning", "Please select a country first.")
            return

        self.progress_bar.start()
        song = self.entry_search_song.get()
        # Perform song search
        search_results = self.database.search_songs(song)

        # Create a new window to display search results
        search_window = tk.Toplevel(self.root)
        search_window.title("Search Results")

        search_results_text = tk.Text(search_window, height=10, width=40)
        search_results_text.pack(padx=10, pady=10)

        # Insert search results into the text widget
        for title, url in search_results:
            search_results_text.insert(tk.END, f"{title}\nURL: {url}\n\n")

        # Create a "Back" button to close the search window and go back to the main screen
        back_button = tk.Button(search_window, text="Back", command=search_window.destroy)
        back_button.pack(pady=5)

        self.progress_bar.stop()

    def start(self):
        self.combobox_country['values'] = self.database.get_unique_countries()
        self.root.mainloop()
